/*
** Haaf's Game Engine 1.5
** Copyright (C) 2003-2005, Relish Games
** hge.relishgames.com
**
** hge_tut03 - Using helper classes
*/


// Copy the files "particles.png", "menu.wav",
// "font1.fnt", "font1.png" and "trail.psi" from
// the folder "precompiled" to the folder with
// executable file. Also copy hge.dll and bass.dll
// to the same folder.


#include "..\..\include\hge.h"
#include "..\..\include\hgesprite.h"
#include "..\..\include\hgefont.h"
#include "..\..\include\hgeparticle.h"

#include <vector>
#include <list>
#include <map>

struct CPt
{
	float m_fX;
	float m_fY;
};

struct CPlayer
{
	CPlayer()
	:
	m_iPoints( 0 )
	{};

	int m_iPoints;

	float m_fX;
	float m_fY;

	float m_fDir;

	float m_fSpeed;
	float m_fMaxSpeed;

	float m_fAnimFrame;

	bool m_bDead;
	bool m_bReallyDead;

	int m_iGamePause;

	int m_iKeyLeft;
	int m_iKeyRight;

	DWORD m_dwColor;

	std::list< CPt > m_liPoints;

	int m_iMaxPoints;
	CPt m_ptLast;

	int m_iExtraHard;
	int m_iHole;

};

enum ExtraType
{
	ET_SIMPLE,
	ET_SPEED
};

struct CExtra
{
	CExtra()
	:
	m_fAnim( 0.0f )
	{};
	CPt m_ptPos;
	ExtraType m_Type;
	float m_fAnim;
};

std::vector< CPlayer > g_MultiPlayer;

std::vector< CExtra > g_Extras;

std::vector< CPt > g_vecHardPoints;

std::vector< CPt > g_vecGrassPoints;

const float g_fSizeX= 1024.0f;
const float g_fSizeY= 768.0f;
const float g_fPlayerSize= 8.0f;

const float g_fPointSize= 6.0f;
const float g_fPointDist= 18.0f;

enum GameMode
{
	GM_KEYMODE,
	GM_RUNNING,
	GM_GAMEOVER
};

GameMode g_Mode;

// Pointer to the HGE interface.
// Helper classes require this to work.
HGE *hge=0;


HEFFECT				sndDie;
HEFFECT				sndWin;
HEFFECT				sndSkull;
HEFFECT				sndSpeed;

HMUSIC				mscBack;

void addPoints()
{
	int iLeft= 0;
	for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
	{
		CPlayer& p= g_MultiPlayer[ i ];
		if( !p.m_bDead )
		{
			p.m_iPoints+= 5;
			iLeft++;
		}
	}

	if( iLeft == 1 )
	{
		hge->Effect_Play( sndWin );
	}
	else
	{
		hge->Effect_Play( sndDie );
	}

}

void setupPlayers()
{
	g_Extras.clear();
	g_vecHardPoints.clear();

	g_vecGrassPoints.clear();
	int iLast= rand() % 12 + 18;
	for( int i= 0; i < iLast; ++i )
	{
		CPt ptNew;
		ptNew.m_fX= (float)( rand() % 1024 );
		ptNew.m_fY= (float)( rand() % 768 );
		g_vecGrassPoints.push_back( ptNew );
	}
	for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
	{
		CPlayer& p= g_MultiPlayer[ i ];
		
		bool bOk;
		do
		{
			p.m_fX= ( rand() % 800 ) + 112.0f;
			p.m_fY= ( rand() % ( 768 - 224 ) ) + 112.0f;

			p.m_fDir= (float)( rand() % 628 ) * 0.01f;

			bOk= true;
			for( int j= 0; j < i; ++j )
			{
				CPlayer& pAct= g_MultiPlayer[ j ];

				float fDistX= p.m_fX - pAct.m_fX;
				float fDistY= p.m_fY - pAct.m_fY;
				float fDist= fDistX * fDistX + fDistY * fDistY;
				if( fDist < 100.0f * 100.0f )
				{
					bOk= false;
				}
			}
		} while ( !bOk );
		
		//p.m_fDir= 0.0f;

		p.m_fSpeed= 1.0f;
		p.m_fMaxSpeed= 1.3f;

		p.m_fAnimFrame= 0.0f;

		p.m_bDead= true;
		p.m_bReallyDead= false;

		p.m_iGamePause= 50;

		p.m_liPoints.clear();

		p.m_ptLast.m_fX= p.m_fX;
		p.m_ptLast.m_fY= p.m_fY;
		p.m_iMaxPoints= 20;

		p.m_iExtraHard= 0;
		p.m_iHole= 0;

		switch( i )
		{
		case 1:
			p.m_iKeyLeft= HGEK_CTRL;
			p.m_iKeyRight= HGEK_ALT;
			p.m_dwColor= 0xff6060ff;
			break;
		case 5:
			p.m_iKeyLeft= HGEK_LBUTTON;
			p.m_iKeyRight= HGEK_RBUTTON;
			p.m_dwColor= 0xff40ff40;
			break;
		case 0:
			p.m_iKeyLeft= HGEK_1;
			p.m_iKeyRight= HGEK_Q;
			p.m_dwColor= 0xffffff00;
			break;
		case 2:
			p.m_iKeyLeft= HGEK_M;
			p.m_iKeyRight= HGEK_COMMA;
			p.m_dwColor= 0xff40ffb0;
			break;
		case 3:
			p.m_iKeyLeft= HGEK_LEFT;
			p.m_iKeyRight= HGEK_DOWN;
			p.m_dwColor= 0xffff00ff;
			break;
		case 4:
			p.m_iKeyLeft= HGEK_DIVIDE;
			p.m_iKeyRight= HGEK_MULTIPLY;
			p.m_dwColor= 0xffff4040;
			break;
		}
	}
}

void testHeads( CPlayer& p )
{
	float fDistSum = g_fPointSize + g_fPlayerSize;

	for( int j= 0; j < (int)g_MultiPlayer.size(); ++j )
	{
		CPlayer pAct= g_MultiPlayer[ j ];
		for(	std::list< CPt >::iterator it= pAct.m_liPoints.begin();
				it != pAct.m_liPoints.end();
				++it )
		{
			float fDistX= p.m_fX - it->m_fX;
			float fDistY= p.m_fY - it->m_fY;
			float fDist= fDistX * fDistX + fDistY * fDistY;
			if( fDist < fDistSum * fDistSum )
			{
				p.m_bDead= true;
				p.m_bReallyDead= true;
				addPoints();
				return;
			}
		}
	}

	for(	std::vector< CPt >::iterator it= g_vecHardPoints.begin();
			it != g_vecHardPoints.end();
			++it )
	{
			float fDistX= p.m_fX - it->m_fX;
			float fDistY= p.m_fY - it->m_fY;
			float fDist= fDistX * fDistX + fDistY * fDistY;
			if( fDist < fDistSum * fDistSum )
			{
				p.m_bDead= true;
				p.m_bReallyDead= true;
				addPoints();
				return;
			}
	}
}

int stepPlayers()
{
	{
		static int iExtraCount= 0;
		iExtraCount++;
		if( iExtraCount == 300 )
		{
			iExtraCount= 0;

			CExtra newExtra;
			
			newExtra.m_ptPos.m_fX= ( rand() % 800 ) + 112.0f;
			newExtra.m_ptPos.m_fY= ( rand() % ( 768 - 224 ) ) + 112.0f;

			newExtra.m_Type= ET_SIMPLE;

			if( rand() % 4 == 2 )
			{
				newExtra.m_Type= ET_SPEED;
			}
			g_Extras.push_back( newExtra );
		}
		
	}
	int iAlive= 0;

	for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
	{
		CPlayer& p= g_MultiPlayer[ i ];
		if( p.m_bDead )
		{
			continue;
		}

		float fAngle= 0.025f;
		float fSpeedStep= 0.001f * p.m_fMaxSpeed;
		if (hge->Input_GetKeyState(p.m_iKeyLeft))
		{
			p.m_fDir-= fAngle;
			p.m_fSpeed-= 2.0f * fSpeedStep;
		}
		if (hge->Input_GetKeyState(p.m_iKeyRight))
		{
			p.m_fDir+= fAngle;
			p.m_fSpeed-= 2.0f * fSpeedStep;
		}

		if( p.m_iGamePause > 0 )
		{
			iAlive++;
			p.m_iGamePause--;
			continue;
		}

		float fDistSum = g_fPointSize * 3.0f;

		for( int j= 0; j < (int)g_Extras.size(); ++j )
		{
			CExtra& ex= g_Extras[ j ];
			
			float fDistX= p.m_fX - ex.m_ptPos.m_fX;
			float fDistY= p.m_fY - ex.m_ptPos.m_fY;
			float fDist= fDistX * fDistX + fDistY * fDistY;

			if( fDist < fDistSum * fDistSum )
			{
				switch( ex.m_Type )
				{
				case ET_SIMPLE:
					p.m_iMaxPoints+= 10;
					p.m_iExtraHard+= 10;
					p.m_iPoints+= 1;
					hge->Effect_Play( sndSkull );
					break;
				case ET_SPEED:
					p.m_fMaxSpeed+= 0.1f;
					p.m_fSpeed+= 0.5f;
					p.m_iPoints+= 2;
					hge->Effect_Play( sndSpeed );
					break;
				}

				g_Extras.erase( g_Extras.begin() + j );
				j= 0;

			}
		}

		float dX= sin( p.m_fDir );
		float dY= -cos( p.m_fDir );
		
		p.m_fX+= dX * p.m_fSpeed;
		p.m_fY+= dY * p.m_fSpeed;

		{
			float fDistX= p.m_fX - p.m_ptLast.m_fX;
			float fDistY= p.m_fY - p.m_ptLast.m_fY;
			float fDist= fDistX * fDistX + fDistY * fDistY;
			if( fDist > g_fPointDist * g_fPointDist )
			{
				if( p.m_iExtraHard > 0 )
				{
					p.m_iExtraHard--;
					g_vecHardPoints.push_back( p.m_ptLast );
				}
				else
				{
					if( p.m_iHole > 0 )
					{
						p.m_iHole--;
					}
					else
					{
						p.m_liPoints.push_front( p.m_ptLast );
						if( (int)p.m_liPoints.size() > p.m_iMaxPoints )
						{
							p.m_liPoints.pop_back();
						}
						if( rand() % 50 == 1 )
						{
							//p.m_iHole= 2+ rand() % 2;
						}

					}
				}

				p.m_ptLast.m_fX= p.m_fX;
				p.m_ptLast.m_fY= p.m_fY;
			}
		}
		
		if( p.m_fDir < 0.0f )
		{
			p.m_fDir+= 6.28f;
		}

		//p.m_fDir= 0.0f;

		if( p.m_fSpeed < 0.75f )
		{
			p.m_fSpeed= 0.75f;
		}
		if( p.m_fSpeed < p.m_fMaxSpeed )
		{
			p.m_fSpeed+= fSpeedStep;
		}

		p.m_fAnimFrame+= 0.2f * p.m_fSpeed;

		if( p.m_fX > g_fSizeX - g_fPlayerSize )
		{
			p.m_bDead= true;
			p.m_bReallyDead= true;
			addPoints();
			continue;
		}
		if( p.m_fX < g_fPlayerSize )
		{
			p.m_bDead= true;
			p.m_bReallyDead= true;
			addPoints();
			continue;
		}
		if( p.m_fY > g_fSizeY - g_fPlayerSize )
		{
			p.m_bDead= true;
			p.m_bReallyDead= true;
			addPoints();
			continue;
		}
		if( p.m_fY < g_fPlayerSize )
		{
			p.m_bDead= true;
			p.m_bReallyDead= true;
			addPoints();
			continue;
		}

		testHeads( p );
		if( p.m_bDead )
		{
			continue;
		}

		if( !p.m_bDead )
		{
			iAlive++;
		}
	}

	return iAlive;
}

int keyPlayers( bool bDeactivate= true )
{
	int iAlive= 0;

	for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
	{
		CPlayer& p= g_MultiPlayer[ i ];

		if (hge->Input_GetKeyState(p.m_iKeyLeft))
		{
			if( p.m_bDead )
			{
				if( !p.m_bReallyDead )
				{	
					p.m_bDead= false;
					if( !bDeactivate )
					{	
						p.m_iGamePause= 200;
					}
				}
			}
		}
		if ( hge->Input_GetKeyState(p.m_iKeyRight) && bDeactivate )
		{
			p.m_bDead= bDeactivate;
		}

		if( !p.m_bDead )
		{
			iAlive++;
		}
	}

	return iAlive;
}

// Pointers to the HGE objects we will use
//hgeSprite*			spr;
hgeSprite*			spt;
hgeFont*			fnt;
hgeParticleSystem*	par;

hgeSprite*			sprBack;
hgeSprite*			sprGras;

const int g_PlayerSpriteOffset= 312;
std::vector< hgeSprite* > m_vecPlayerSprites;

std::vector< hgeSprite* > m_vecEnemySprites;

std::vector< hgeSprite* > m_vecNetSprites;


// Handles for HGE resourcces
HTEXTURE			tex;
HTEXTURE			tex2;
HTEXTURE			tex3;
HTEXTURE			texBack;
HTEXTURE			texGras;

// Some "gameplay" variables
float x=100.0f, y=100.0f;
float dx=0.0f, dy=0.0f;

const float speed=90;
const float friction=0.98f;

// Play sound effect
void boom() {
	int pan=int((x-400)/4);
	float pitch=(dx*dx+dy*dy)*0.0005f+0.2f;
	//hge->Effect_PlayEx(snd,100,pan,pitch);
}

float fTime= 0;
float fGameTime= 0;

bool FrameFunc()
{
	float dt=hge->Timer_GetDelta();
	
	fTime+= dt;
	fGameTime+= dt;

	switch( g_Mode )
	{
	case GM_KEYMODE:
		fTime= 0.0f;
		{
			int iPlayers= keyPlayers();
		
			if ( iPlayers > 1 && hge->Input_GetKeyState( HGEK_SPACE ) )
			{
				g_Mode= GM_RUNNING;
				fGameTime= 0.0f;
			}
		}
		break;
	case GM_RUNNING:
		while( fTime > 0.01f )
		{
			fTime-= 0.01f;
			if( stepPlayers() < 2 )
			{
				g_Mode= GM_GAMEOVER;
			}
		}
		break;
	case GM_GAMEOVER:
		//fTime= 0.0f;
		if (hge->Input_GetKeyState( HGEK_ENTER ) )
		{
			setupPlayers();
			g_Mode= GM_KEYMODE;
		}
		break;
	}

	// Process keys
	if (hge->Input_GetKeyState(HGEK_ESCAPE)) return true;
	/*if (hge->Input_GetKeyState(HGEK_LEFT)) dx-=speed*dt;
	if (hge->Input_GetKeyState(HGEK_RIGHT)) dx+=speed*dt;
	if (hge->Input_GetKeyState(HGEK_UP)) dy-=speed*dt;
	if (hge->Input_GetKeyState(HGEK_DOWN)) dy+=speed*dt;*/

	// Do some movement calculations and collision detection	
	/*dx*=friction; dy*=friction; x+=dx; y+=dy;
	if(x>784) {x=784-(x-784);dx=-dx;boom();}
	if(x<16) {x=16+16-x;dx=-dx;boom();}
	if(y>584) {y=584-(y-584);dy=-dy;boom();}
	if(y<16) {y=16+16-y;dy=-dy;boom();}*/

	// Update particle system
	/*par->info.nEmission=(int)(dx*dx+dy*dy)*2;
	par->MoveTo(x,y);
	par->Update(dt);*/

	// Render graphics
	hge->Gfx_BeginScene();
	//hge->Gfx_Clear(0);
	/*par->Render();*/
	//spr->Render(x, y);
	//m_vecPlayerSprites[ g_PlayerSpriteOffset + ( (int)(fTime*10.0f) % 8 ) ]->RenderEx( x, y, 0.0f, 0.5f, 0.5f );

	for( int x= 0; x < 4; ++x )
	{
		for( int y= 0; y < 3; ++y )
		{
			sprBack->Render( x * 256.0f, y * 256.0f );
		}
	}
	for( int i= 0; i < (int)g_vecGrassPoints.size(); ++i )
	{
		sprGras->RenderEx( g_vecGrassPoints[ i ].m_fX, g_vecGrassPoints[ i ].m_fY, i * 0.7853978f );
	}
	for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
	{
		CPlayer& p= g_MultiPlayer[ i ];

		int Frame= (int)( 8.0f * ( p.m_fDir + 6.28f * 0.0f / 16.0f ) / 6.28f );
		Frame= ( Frame % 8 );
		float fFrameDir= (float)Frame * 6.28f / 8.0f;
		Frame= Frame * 8;
		Frame+= ((int)(p.m_fAnimFrame)) % 8;

		DWORD dwColor= p.m_dwColor;

		int iFrameStep= ((int)(p.m_fAnimFrame * 0.5f));
		for(	std::list< CPt >::iterator it= p.m_liPoints.begin();
				it != p.m_liPoints.end();
				++it )
		{
			iFrameStep++;
			iFrameStep%= 8; 
			CPt& pt= *it;
			m_vecEnemySprites[ 80 + iFrameStep * 10 ]->SetColor( dwColor );
			m_vecEnemySprites[ 80 + iFrameStep * 10 ]->RenderEx( 
				pt.m_fX, 
				pt.m_fY, 
				0.0f,
				0.75f, 
				0.75f );
		}

		if( p.m_bDead )
		{
			dwColor= ( dwColor >> 1 ) & 0xff7f7f7f;
		}
		
		m_vecPlayerSprites[ g_PlayerSpriteOffset + Frame ]->SetColor(dwColor);
		m_vecPlayerSprites[ g_PlayerSpriteOffset + Frame ]->RenderEx( 
			p.m_fX, 
			p.m_fY, 
			p.m_fDir - fFrameDir,
			0.75f, 
			0.75f );

	}

	int iFrameStep= 0; //(int)( fTime * 10.0f );	
	for(	std::vector< CPt >::iterator it= g_vecHardPoints.begin();
			it != g_vecHardPoints.end();
			++it )
	{
		iFrameStep++;
		iFrameStep%= 8; 
		CPt& pt= *it;
		m_vecEnemySprites[ 80 + iFrameStep * 10 ]->SetColor( 0xffb0b0b0 );
		m_vecEnemySprites[ 80 + iFrameStep * 10 ]->RenderEx( 
			pt.m_fX, 
			pt.m_fY, 
			0.0f,
			0.75f, 
			0.75f );
	}


	static int iFrameExtra= 0;
	static int iLastSecond= 5000;

	for( int i= 0; i < (int)g_Extras.size(); ++i )
	{
		CExtra& ex= g_Extras[ i ];
		ex.m_fAnim+= dt;
		int iFrame = (int)( ex.m_fAnim * 10.0f );
		switch( ex.m_Type )
		{
		case ET_SIMPLE:
			iFrame%= 8;
			m_vecEnemySprites[ 83 + iFrame * 10 ]->SetColor( 0xffffffff );
			m_vecEnemySprites[ 83 + iFrame * 10 ]->RenderEx( 
				ex.m_ptPos.m_fX, 
				ex.m_ptPos.m_fY,
				0.0f,
				0.85f, 
				0.85f );
			break;
		case ET_SPEED:
			iFrame%= 16;
			m_vecNetSprites[ iFrame ]->SetColor( 0xffffffff );
			m_vecNetSprites[ iFrame ]->RenderEx( 
				ex.m_ptPos.m_fX, 
				ex.m_ptPos.m_fY,
				0.0f,
				1.0f, 
				1.0f );
			break;
		}

	}

	switch( g_Mode )
	{
	case GM_KEYMODE:
		fnt->SetScale( 2.0f );
		fnt->printf( 330, 150, "Tolles Spiel III" );
		fnt->SetScale( 1.0f );

		fnt->printf( 400, 350, "Warte auf Spieler" );
		fnt->printf( 290, 400, "(mindestens 2) - Leertaste zum Start" );

		fnt->printf( 450, 500, "Tasten:" );
		fnt->printf( 115, 550, "Linke Strg;Alt   Maus Links;Rechts   1;Q   M;,   Links;Runter   Numpad *;/" );

		fnt->printf( 650, 725, "Music: Chrome - Evoke Forever" );
		break;
	case GM_RUNNING:
		{
			if( fGameTime < 15.0f )
			{
				
			}
			float fOverTime= ( 45.0f - fGameTime );
			int iSeconds= (int)( fOverTime );
			if( iSeconds < 6 && iSeconds > 0 )
			{
				fnt->printf( 500, 370, "%d", iSeconds );
				if( iSeconds != iLastSecond )
				{
					hge->Effect_Play( sndSkull );
				}
			}
			else if( iSeconds <= 0 )
			{
				if( iSeconds != iLastSecond && iSeconds == 0)
				{
					hge->Effect_Play( sndWin );
				}

				if( iSeconds > -4 )
				{
					int iFrame= 120;
					DWORD dwShade= max( 0, 128 + (int)( fOverTime * 40.0f ) );
					m_vecEnemySprites[ iFrame ]->SetColor( 0xffffff + ( dwShade << 24 ) );
					m_vecEnemySprites[ iFrame ]->RenderEx( 
						512.0f, 
						300.0f, 
						0.0f,
						2.0f + 2.0f * fOverTime * fOverTime, 
						2.0f + 2.0f *  fOverTime * fOverTime );
					fnt->printf( 450, 370, "Sudden Death!" );
				}

				for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
				{
					CPlayer& p= g_MultiPlayer[ i ];
					p.m_iExtraHard= 10;
				}
			}
			else
			{
				keyPlayers( false );
			}

			iLastSecond= iSeconds;
		}
		break;
	case GM_GAMEOVER:
		fnt->printf( 485, 370, "Sieg!" );
		fnt->printf( 440, 700, "Enter - Weiter" );
		//fnt->printf( 170, 550, "Linke Strg/Alt   Maus Links/Rechts   1/Q   M/,   Links/Runter" );
		break;
	}

	for( int i= 0; i < (int)g_MultiPlayer.size(); ++i )
	{
		CPlayer& p= g_MultiPlayer[ i ];
		DWORD dwColor= p.m_dwColor;
		//dwColor= ( dwColor >> 1 ) & 0x7f7f7f ) | 0xff000000;

		int iGreater= 0;
		for( int j= 0; j < (int)g_MultiPlayer.size(); ++j )
		{
			CPlayer& pAct= g_MultiPlayer[ j ];
			if( pAct.m_iPoints > p.m_iPoints )
			{
				iGreater++;
			}
		}

		if( g_Mode == GM_GAMEOVER && iGreater == 0)
		{
			int iAdd= (int)( ( 0.15f + 0.25f * sinf( fTime * 5.0f ) ) * 255.0f );
			int dwRed= ( dwColor >> 16 ) & 0xff;
			int dwGreen= ( dwColor >> 8 ) & 0xff;
			int dwBlue= dwColor & 0xff;
			dwRed= max( 0, min( 255, dwRed - iAdd ) );
			dwGreen= max( 0, min( 255, dwGreen - iAdd ) );
			dwBlue= max( 0, min( 255, dwBlue - iAdd ) );

			dwColor=	0xff000000 |
						dwRed << 16 |
						dwGreen << 8 |
						dwBlue;
		}
		fnt->SetColor( dwColor ); 
		fnt->printf( i * 100.0f + 250.0f, 5.0f, "%d", p.m_iPoints);
		fnt->SetScale( 1.0f );
	}
	fnt->SetColor( 0xa0ffffff );

	//fnt->printf(5,5,"dt:%.3f\nFPS:%d (constant)", dt, hge->Timer_GetFPS());
	hge->Gfx_EndScene();

	return false;
}


int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
{
	srand( GetTickCount() );
	g_Mode= GM_KEYMODE;

	g_MultiPlayer.resize( 6 );
	setupPlayers();

	hge = hgeCreate(HGE_VERSION);

	hge->System_SetState(HGE_LOGFILE, "ich.log");
	hge->System_SetState(HGE_FRAMEFUNC, FrameFunc);
	hge->System_SetState(HGE_TITLE, "Tolles Spiel III - geht voll ab!");
	hge->System_SetState(HGE_FPS, 100);
	hge->System_SetState(HGE_WINDOWED, false );
	hge->System_SetState(HGE_SCREENWIDTH, 1024);
	hge->System_SetState(HGE_SCREENHEIGHT, 768);
	hge->System_SetState(HGE_SCREENBPP, 32);

	if(hge->System_Initiate()) {

		// Load sound and texture
		sndDie=hge->Effect_Load("scream.wav");
		sndWin=hge->Effect_Load("manwah.wav");
		sndSpeed=hge->Effect_Load("speed.wav");
		sndSkull=hge->Effect_Load("squeaky.wav");

		mscBack= hge->Music_Load("music.xm");

		tex= hge->Texture_Load("redspider96x96.tga");
		texBack=  hge->Texture_Load("back.png");
		texGras=  hge->Texture_Load("gras.png");
		tex2= hge->Texture_Load("floatingskull48x64.tga");
		tex3= hge->Texture_Load("netammo32x32.tga");


		for( int y= 0; y < 21; ++y )
		{
			for( int x= 0; x < 21; ++x )
			{
				hgeSprite* spr;
				spr=new hgeSprite(tex,  2 + 96 * (float)x, 2 + 96 * (float)y, 96 - 4 , 96 - 4);
				//spr->SetColor(0xFFFFA000);
				spr->SetHotSpot(46,46);
				m_vecPlayerSprites.push_back( spr );
			}
		}

		for( int y= 0; y < 8; ++y )
		{
			for( int x= 0; x < 21; ++x )
			{
				hgeSprite* spr;
				spr=new hgeSprite(tex2,  2 + 48 * (float)x, 2 + 64 * (float)y, 48 - 4 , 64 - 4);
				//spr->SetColor(0xFFFFA000);
				spr->SetHotSpot(22,30);
				m_vecEnemySprites.push_back( spr );
			}
		}

		for( int y= 0; y < 4; ++y )
		{
			for( int x= 0; x < 8; ++x )
			{
				hgeSprite* spr;
				spr=new hgeSprite(tex3,  2 + 32 * (float)x, 2 + 32 * (float)y, 32 - 4 , 32 - 4);
				//spr->SetColor(0xFFFFA000);
				spr->SetHotSpot(14,14);
				m_vecNetSprites.push_back( spr );
			}
		}

		sprBack= new hgeSprite(texBack, 0, 0, 256, 256 );
		sprGras= new hgeSprite(texGras, 0, 0, 128, 128 );
		sprGras->SetHotSpot( 64.0f, 64.0f );


		// Load a font
		fnt=new hgeFont("font1.fnt");
		fnt->SetColor( 0xa0ffffff );

		hge->Music_Play( mscBack, true );

		// Create and set up a particle system
		spt=new hgeSprite(tex, 32, 32, 32, 32);
		spt->SetBlendMode(BLEND_COLORMUL | BLEND_ALPHAADD | BLEND_NOZWRITE);
		spt->SetHotSpot(48,48);
		/*par=new hgeParticleSystem("trail.psi",spt);
		par->Fire();*/

		// Let's rock now!
		hge->System_Start();

		// Delete created objects and free loaded resources
		//delete par;
		delete fnt;
		delete spt;
		//delete spr;
		hge->Texture_Free(tex);
		//hge->Effect_Free(snd);
	}

	// Clean up and shutdown
	hge->System_Shutdown();
	hge->Release();
	return 0;
}
